@@ -17,6 +17,7 @@
 #include <KStandardAction>
 
 #include <QFileDialog>
+#include <QRegularExpression>
 #include <QStandardPaths>
 #include <QTimer>
 
@@ -71,9 +72,9 @@ void Game::initGame (GameGLView * glv, Kubrick * mw)
     gameGLView = glv;			// Save OpenGL view, used when drawing.
     mainWindow = mw;			// Save main window, shows status, etc.
 
-    // mainWindow->setToggle ("toggle_tumbling", tumbling);
-    mainWindow->setToggle ("watch_shuffling", (bool) option [optViewShuffle]);
-    mainWindow->setToggle ("watch_moves",     (bool) option [optViewMoves]);
+    // mainWindow->setToggle (QStringLiteral("toggle_tumbling"), tumbling);
+    mainWindow->setToggle (QStringLiteral("watch_shuffling"), (bool) option [optViewShuffle]);
+    mainWindow->setToggle (QStringLiteral("watch_moves"),     (bool) option [optViewMoves]);
     gameGLView->setBevelAmount (option [optBevel]);
     gameGLView->setCursor (Qt::CrossCursor);
 
@@ -169,7 +170,7 @@ void Game::load ()
 
     KConfig config (loadFilename, KConfig::SimpleConfig);
 
-    if (! config.hasGroup ("KubrickGame")) {
+    if (! config.hasGroup (QStringLiteral("KubrickGame"))) {
 	KMessageBox::error(mainWindow,
                            i18n("The file '%1' is not a valid Kubrick game-file.", loadFilename));
 	return;
@@ -206,8 +207,8 @@ void Game::changePuzzle (const Kubrick::PuzzleItem & puzzle)
     option [optShuffleMoves] = puzzle.shuffleMoves;
     option [optViewShuffle]  = puzzle.viewShuffle;
     option [optViewMoves]    = puzzle.viewMoves;
-    mainWindow->setToggle ("watch_shuffling", (bool) option [optViewShuffle]);
-    mainWindow->setToggle ("watch_moves",     (bool) option [optViewMoves]);
+    mainWindow->setToggle (QStringLiteral("watch_shuffling"), (bool) option [optViewShuffle]);
+    mainWindow->setToggle (QStringLiteral("watch_moves"),     (bool) option [optViewMoves]);
 
     // Build the new cube and shuffle it.
     newCube (option [optXDim], option [optYDim], option [optZDim],
@@ -321,9 +322,9 @@ void Game::setStandardView()
     // The series of moves is delimited by spaces in the Singmaster display.
 
     if ((singmasterString.length() > 0) &&
-	(singmasterString.right (1) != &(SingmasterNotation [SM_SPACER])))
+	(singmasterString.right (1) != QLatin1Char(SingmasterNotation [SM_SPACER])))
     {
-	singmasterString.append (SingmasterNotation [SM_SPACER]);
+	singmasterString.append (QLatin1Char(SingmasterNotation [SM_SPACER]));
     }
 
     while (! tempMoves.isEmpty()) {
@@ -336,7 +337,7 @@ void Game::setStandardView()
 	singmasterString.append (s);
     }
 
-    singmasterString.append (SingmasterNotation [SM_SPACER]);
+    singmasterString.append (QLatin1Char(SingmasterNotation [SM_SPACER]));
     smSelectionLength++;
 
     // Update the Singmaster Moves display.
@@ -346,20 +347,20 @@ void Game::setStandardView()
 
 void Game::undoAll ()
 {
-    startUndo (QChar('U'), i18n("Restart Puzzle (Undo All)"));
+    startUndo (QStringLiteral("U"), i18n("Restart Puzzle (Undo All)"));
 }
 
 
 void Game::redoAll ()
 {
-    startRedo (QChar('R'), i18n("Redo All"));
+    startRedo (QStringLiteral("R"), i18n("Redo All"));
 }
 
 
 void Game::changeScene (const int newSceneID)
 {
-    QString sceneActionName = "scene_" + QString::number(newSceneID);
-    mainWindow->setToggle (sceneActionName.toLatin1().data(), true);
+    QString sceneActionName = QStringLiteral("scene_%1").arg(newSceneID);
+    mainWindow->setToggle (sceneActionName, true);
 
     currentSceneID = newSceneID;
     option [optSceneID] = currentSceneID;
@@ -404,7 +405,7 @@ void Game::loadDemo (const QString & file)
 	return;
     QString demoFile = QStandardPaths::locate(QStandardPaths::AppDataLocation, file);
     KConfig config (demoFile, KConfig::SimpleConfig);
-    if (config.hasGroup ("KubrickGame")) {
+    if (config.hasGroup (QStringLiteral("KubrickGame"))) {
 	if (! demoPhase) {
 	    saveState ();
 	    startDemo ();
@@ -549,7 +550,7 @@ void Game::addPlayersMove (Move * move)
     singmasterString.append (tempString);
 
     // Trigger the animation's advance() cycle to do the move(s).
-    startAnimation (displaySequence + QChar('m'), option [optSceneID],
+    startAnimation (displaySequence + QLatin1Char('m'), option [optSceneID],
 			option [optViewShuffle], option [optViewMoves]);
 }
 
@@ -651,7 +652,7 @@ void Game::smWaitingForInput (const SingmasterMove smCode)
     switch (smCode) {
     case SM_INNER:
 	smDotCount++;
-	smTempString.append (SingmasterNotation [SM_INNER]);
+	smTempString.append (QLatin1Char(SingmasterNotation [SM_INNER]));
 	keyboardState = SingmasterPrefixSeen;	// Change the state.
 	break;
     case SM_ANTICLOCKWISE:
@@ -663,7 +664,7 @@ void Game::smWaitingForInput (const SingmasterMove smCode)
 	keyboardState = WaitingForInput;
 	break;
     case SM_SPACER:
-	singmasterString.append (SingmasterNotation [SM_SPACER]);
+	singmasterString.append (QLatin1Char(SingmasterNotation [SM_SPACER]));
 	keyboardState = WaitingForInput;	// No change of state.
 	break;
     case SM_UP:
@@ -687,7 +688,7 @@ void Game::smSingmasterPrefixSeen (const SingmasterMove smCode)
     switch (smCode) {
     case SM_INNER:
 	smDotCount++;
-	smTempString.append (SingmasterNotation [SM_INNER]);
+	smTempString.append (QLatin1Char(SingmasterNotation [SM_INNER]));
 	keyboardState = SingmasterPrefixSeen;	// No change of state.
 	break;
     case SM_ANTICLOCKWISE:
@@ -721,7 +722,7 @@ void Game::smSingmasterFaceIDSeen (const SingmasterMove smCode)
     case SM_INNER:
 	executeSingmasterMove (SM_EXECUTE);
 	smDotCount = 1;
-	smTempString = SingmasterNotation [SM_INNER];
+	smTempString = QLatin1Char(SingmasterNotation [SM_INNER]);
 	keyboardState = SingmasterPrefixSeen;	// Change the state.
 	break;
     case SM_ANTICLOCKWISE:
@@ -799,7 +800,7 @@ void Game::saveSingmasterFaceID (const SingmasterMove smCode)
 
     // Edit the temporary Singmaster move-string and add the move-notation.
     smTempString = (smDotCount > 0) ? smTempString.left (smDotCount) : QString();
-    smTempString.append (SingmasterNotation [smCode]);
+    smTempString.append (QLatin1Char(SingmasterNotation [smCode]));
 
     // An invisible face (D, L, B) will be slice 1 and a visible face will have
     // a slice number the same as the cube dimension.  This is adjusted up or
@@ -827,12 +828,12 @@ void Game::executeSingmasterMove (const SingmasterMove smCode)
 {
     switch (smCode) {
     case SM_ANTICLOCKWISE:
-	smTempString.append (SingmasterNotation [SM_ANTICLOCKWISE]);
+	smTempString.append (QLatin1Char(SingmasterNotation [SM_ANTICLOCKWISE]));
 	smMoveDirection = (smMoveDirection == CLOCKWISE) ?
 				ANTICLOCKWISE : CLOCKWISE;
 	break;
     case SM_DOUBLE:
-	smTempString.append (SingmasterNotation [SM_DOUBLE]);
+	smTempString.append (QLatin1Char(SingmasterNotation [SM_DOUBLE]));
 	smMoveDirection = ONE_EIGHTY;
 	break;
     case SM_2_SLICE:
@@ -840,7 +841,7 @@ void Game::executeSingmasterMove (const SingmasterMove smCode)
     case SM_EXECUTE:
 	break;
     case SM_SPACER:
-	smTempString.append (SingmasterNotation [SM_SPACER]);
+	smTempString.append (QLatin1Char(SingmasterNotation [SM_SPACER]));
 	break;
     default:
 	qCDebug(KUBRICK_LOG) << "'Impossible' Singmaster code" << smCode;
@@ -891,10 +892,10 @@ QString Game::convertMoveToSingmaster (const Move * move)
     int     direction = move->direction;
 
     if (move->slice == WHOLE_CUBE) {
-	dots = SingmasterNotation [SM_CUBE];
+	dots = QLatin1Char(SingmasterNotation [SM_CUBE]);
     }
     else {
-	dots.fill (QChar(SingmasterNotation [SM_INNER]), cubeSize [move->axis]);
+	dots.fill (QLatin1Char(SingmasterNotation [SM_INNER]), cubeSize [move->axis]);
 	dots = (move->slice < 0) ? dots.left (slice - 1) :
 			       dots.left (cubeSize [move->axis] - slice);
     }
@@ -903,8 +904,8 @@ QString Game::convertMoveToSingmaster (const Move * move)
 	direction = (direction == CLOCKWISE) ? ANTICLOCKWISE : CLOCKWISE;
     }
 
-    QString smMove = dots + SingmasterNotation [s] + ((direction == CLOCKWISE) ?
-            QString() : QString (SingmasterNotation [SM_ANTICLOCKWISE]));
+    QString smMove = dots + QLatin1Char(SingmasterNotation [s]) + ((direction == CLOCKWISE) ?
+            QString() : QString (QLatin1Char(SingmasterNotation [SM_ANTICLOCKWISE])));
     return smMove;
 }
 
@@ -958,7 +959,7 @@ void Game::restoreState ()
 {
     const QString rFile = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + QLatin1Char('/') + QStringLiteral("kubrick.save");
     KConfig config (rFile, KConfig::SimpleConfig);
-    if (config.hasGroup ("KubrickGame")) {
+    if (config.hasGroup (QStringLiteral("KubrickGame"))) {
 	loadPuzzle (config);
     }
     else {
@@ -1057,8 +1058,8 @@ int Game::doOptionsDialog (bool changePuzzle) // Private function.
 	gameGLView->setBevelAmount (option [optBevel]);
 	viewShuffle = (bool) option [optViewShuffle];
 	viewMoves = (bool) option [optViewMoves];
-	mainWindow->setToggle ("watch_shuffling", (bool)option[optViewShuffle]);
-	mainWindow->setToggle ("watch_moves",     (bool)option[optViewMoves]);
+	mainWindow->setToggle (QStringLiteral("watch_shuffling"), (bool)option[optViewShuffle]);
+	mainWindow->setToggle (QStringLiteral("watch_moves"),     (bool)option[optViewMoves]);
     }
 
     delete d;
@@ -1069,6 +1070,9 @@ int Game::doOptionsDialog (bool changePuzzle) // Private function.
 
 void Game::drawScene ()
 {
+    // Save data with QOpenGLWidget only valid during paintGL()
+    moveTracker->saveSceneInfo();
+
     // The size of the cubes on-screen is determined by the height of the
     // window and the OpenGL perspective projection, in which the window-height
     // always has a set angle of view.  The lateral spacing of the cubes (X
@@ -1173,10 +1177,10 @@ void Game::tumble ()
 void Game::startDemo ()
 {
     // Set the demo's toggle-button ON.
-    mainWindow->setToggle (KStandardGameAction::name (KStandardGameAction::Demo), true);
+    mainWindow->setToggle (KGameStandardAction::name (KGameStandardAction::Demo), true);
     // Disable the Save actions.
-    mainWindow->setAvail (KStandardGameAction::name (KStandardGameAction::Save),   false);
-    mainWindow->setAvail (KStandardGameAction::name (KStandardGameAction::SaveAs), false);
+    mainWindow->setAvail (KGameStandardAction::name (KGameStandardAction::Save),   false);
+    mainWindow->setAvail (KGameStandardAction::name (KGameStandardAction::SaveAs), false);
     // Disable the Preferences action.
     mainWindow->setAvail (KStandardAction::name (KStandardAction::Preferences), false);
 
@@ -1228,15 +1232,15 @@ void Game::randomDemo ()
 void Game::stopDemo ()
 {
     // Set the demo's toggle-button OFF.
-    mainWindow->setToggle (KStandardGameAction::name (KStandardGameAction::Demo), false);
+    mainWindow->setToggle (KGameStandardAction::name (KGameStandardAction::Demo), false);
     // Enable the Save actions.
-    mainWindow->setAvail (KStandardGameAction::name (KStandardGameAction::Save),   true);
-    mainWindow->setAvail (KStandardGameAction::name (KStandardGameAction::SaveAs), true);
+    mainWindow->setAvail (KGameStandardAction::name (KGameStandardAction::Save),   true);
+    mainWindow->setAvail (KGameStandardAction::name (KGameStandardAction::SaveAs), true);
     // Enable the Preferences action.
     mainWindow->setAvail (KStandardAction::name (KStandardAction::Preferences), true);
 
     tumbling = false;
-    // mainWindow->setToggle ("toggle_tumbling", tumbling);
+    // mainWindow->setToggle (QStringLiteral("toggle_tumbling"), tumbling);
 
     demoL->setVisible (false);		// Hide the DEMO text.
     demoPhase = false;
@@ -1363,8 +1367,8 @@ void Game::savePuzzle (KConfig & config)
     }
 
     // Clear any previously saved info (in case there are fewer moves now).
-    config.deleteGroup ("KubrickGame");
-    KConfigGroup configGroup = config.group("KubrickGame");
+    config.deleteGroup (QStringLiteral("KubrickGame"));
+    KConfigGroup configGroup = config.group(QStringLiteral("KubrickGame"));
 
     QStringList list;
 
@@ -1382,7 +1386,7 @@ void Game::savePuzzle (KConfig & config)
     list.clear ();
     list.append (QString::asprintf ("%d", shuffleMoves));
     list.append (QString::asprintf ("%d", playerMoves));
-    list.append (QString::asprintf ("%d", moves.count()));
+    list.append (QString::asprintf ("%" PRIdQSIZETYPE, moves.count()));
     configGroup.writeEntry ("f) MoveCounts", list);
 
     // Save the list of Singmaster moves.
@@ -1417,7 +1421,7 @@ void Game::loadPuzzle (KConfig & config)
     QList<Move *> movesTemp;
     Move * moveTemp;
 
-    KConfigGroup configGroup = config.group ("KubrickGame");
+    KConfigGroup configGroup = config.group (QStringLiteral("KubrickGame"));
 
     list = configGroup.readEntry ("a) Options", notFound);
     int nOpt = 0;
@@ -1465,9 +1469,9 @@ void Game::loadPuzzle (KConfig & config)
     tumbling		= option [optTumbling];
     tumblingTicks	= option [optTumblingTicks];
     if (mainWindow != nullptr) {
-	// mainWindow->setToggle ("toggle_tumbling", tumbling);
-	mainWindow->setToggle ("watch_shuffling", (bool)option[optViewShuffle]);
-	mainWindow->setToggle ("watch_moves",     (bool)option[optViewMoves]);
+	// mainWindow->setToggle (QStringLiteral("toggle_tumbling"), tumbling);
+	mainWindow->setToggle (QStringLiteral("watch_shuffling"), (bool)option[optViewShuffle]);
+	mainWindow->setToggle (QStringLiteral("watch_moves"),     (bool)option[optViewMoves]);
     }
 
     qDeleteAll(moves);
@@ -1784,7 +1788,7 @@ void Game::advance()
     // we do not do this in the advance() method itself.  This is not essential,
     // but makes the game-logic and rendering more independent of each other.
 
-    QTimer::singleShot(0, gameGLView, &GameGLView::updateGL);
+    QTimer::singleShot(0, gameGLView, qOverload<>(&GameGLView::update));
 }
 
 
@@ -1892,7 +1896,7 @@ void Game::startAnimatedMove (Move * move, int speed)
 	// The "-" in the pattern must be just before the "]", otherwise it
 	// defines a range of characters and does not get matched as a "-".
 
-    QRegExp smPattern (QStringLiteral("[.C]*[FBLRUD]['2 +-]*"));
+	QRegularExpression smPattern (QStringLiteral("[.C]*[FBLRUD]['2 +-]*"));
 	// IDW testing - qCDebug(KUBRICK_LOG) << "Undoing" << undoing << singmasterString <<
 			// IDW testing - smSelectionStart << smSelectionLength;
 	if (undoing) {
@@ -1901,20 +1905,22 @@ void Game::startAnimatedMove (Move * move, int speed)
 	    smSelectionStart = 0;
 	    smSelectionLength = 0;
 	    while (smSelectionStart < pos2) {
-		pos1 = smPattern.indexIn (singmasterString, pos1);
+		QRegularExpressionMatch match = smPattern.match(singmasterString, pos1);
+		pos1 = match.capturedStart();
 		if ((pos1 >= pos2) || (pos1 < 0)) {
 		    break;
 		}
 		smSelectionStart = pos1;
-		smSelectionLength = smPattern.matchedLength();
+		smSelectionLength = match.capturedLength();
 		pos1 = pos1 + smSelectionLength;
 	    }
 	}
 	else {
 	    int pos1 = smSelectionStart + smSelectionLength;
-	    int pos2 = smPattern.indexIn (singmasterString, pos1);
+	    QRegularExpressionMatch match = smPattern.match(singmasterString, pos1);
+	    int pos2 = match.capturedStart();
 	    if (pos2 >= 0) {
-		pos2 = pos2 + smPattern.matchedLength();
+		pos2 = pos2 + match.capturedLength();
 		smSelectionStart = pos1;
 		smSelectionLength = pos2 - pos1;
 	    }
